@@ -1,3 +1,5 @@
+# -*- coding: utf-8 -*-
+
 # ***** BEGIN LICENSE BLOCK *****
 # This Source Code Form is subject to the terms of the Mozilla Public
 # License, v. 2.0. If a copy of the MPL was not distributed with this file,
@@ -13,13 +15,15 @@
 # ***** END LICENSE BLOCK *****
 from __future__ import absolute_import
 from datetime import datetime
-from metlog.client import MetlogClient
+from metlog.client import MetlogClient, SEVERITY
 from mock import Mock
 from nose.tools import eq_, ok_
 from metlog.senders.dev import DebugCaptureSender
 
-import socket
+import StringIO
 import os
+import socket
+import sys
 import threading
 import time
 
@@ -83,6 +87,65 @@ class TestMetlogClient(object):
                             'timestamp': metlog_args['timestamp'].isoformat()})
         eq_(actual_msg, metlog_args)
 
+    def test_oldstyle(self):
+        payload = 'debug message'
+        self.client.debug(payload)
+        full_msg = self._extract_full_msg()
+        eq_(full_msg['payload'], payload)
+        eq_(full_msg['severity'], SEVERITY.DEBUG)
+
+    def test_oldstyle_args(self):
+        payload = '1, 2: %s\n3, 4: %s'
+        args = ('buckle my shoe', 'shut the door')
+        self.client.warn(payload, *args)
+        full_msg = self._extract_full_msg()
+        eq_(full_msg['payload'], payload % args)
+
+    def test_oldstyle_mapping_arg(self):
+        payload = '1, 2: %(onetwo)s\n3, 4: %(threefour)s'
+        args = {'onetwo': 'buckle my shoe',
+                'threefour': 'shut the door'}
+        self.client.warn(payload, args)
+        full_msg = self._extract_full_msg()
+        eq_(full_msg['payload'], payload % args)
+
+    def test_oldstyle_exc_info(self):
+        payload = 'traceback ahead -->'
+        try:
+            a = b  # NOQA
+        except NameError:
+            self.client.error(payload, exc_info=True)
+        full_msg = self._extract_full_msg()
+        ok_(full_msg['payload'].startswith(payload))
+        ok_("NameError: global name 'b' is not defined" in full_msg['payload'])
+        ok_('test_client.py' in full_msg['payload'])
+
+    def test_oldstyle_exc_info_auto(self):
+        payload = 'traceback ahead -->'
+        try:
+            a = b  # NOQA
+        except NameError:
+            self.client.exception(payload)
+        full_msg = self._extract_full_msg()
+        ok_(full_msg['payload'].startswith(payload))
+        ok_("NameError: global name 'b' is not defined" in full_msg['payload'])
+        ok_('test_client.py' in full_msg['payload'])
+
+    def test_oldstyle_exc_info_passed(self):
+        def name_error():
+            try:
+                a = b  # NOQA
+            except NameError:
+                return sys.exc_info()
+
+        ei = name_error()
+        payload = 'traceback ahead -->'
+        self.client.critical(payload, exc_info=ei)
+        full_msg = self._extract_full_msg()
+        ok_(full_msg['payload'].startswith(payload))
+        ok_("NameError: global name 'b' is not defined" in full_msg['payload'])
+        ok_('test_client.py' in full_msg['payload'])
+
     def test_timer_contextmanager(self):
         name = self.timer_name
         with self.client.timer(name) as timer:
@@ -258,3 +321,33 @@ class TestDisabledTimer(object):
         foo2()
 
         eq_(len(self.mock_sender.msgs), 0)
+
+
+class TestUnicode(object):
+    logger = 'tests'
+    timer_name = 'test'
+
+    def setUp(self):
+        self.mock_sender = Mock()
+        self.mock_sender.send_message.side_effect = \
+                UnicodeError("UnicodeError encoding user data")
+        self.client = MetlogClient(self.mock_sender, self.logger)
+        # overwrite the class-wide threadlocal w/ an instance one
+        # so values won't persist btn tests
+        self.timer_ob = self.client.timer(self.timer_name)
+        self.timer_ob.__dict__['_local'] = threading.local()
+
+        self.old_stderr = sys.stderr
+        sys.stderr = StringIO.StringIO()
+
+    def tearDown(self):
+        del self.timer_ob.__dict__['_local']
+        del self.mock_sender
+        sys.stderr = self.old_stderr
+
+    def test_unicode_failure(self):
+        msg = "mock will raise unicode error here"
+        self.client.send_message(msg)
+        sys.stderr.seek(0)
+        err = sys.stderr.read()
+        ok_('Error sending' in err)
